@@ -1,8 +1,7 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8; coding: utf-8 -*-
- * gtksourcegutterrenderertext.c
+/*
  * This file is part of GtkSourceView
  *
- * Copyright (C) 2010 - Jesse van den Kieboom
+ * Copyright 2010 - Jesse van den Kieboom
  *
  * GtkSourceView is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,12 +17,9 @@
  * along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "config.h"
 
 #include "gtksourcegutterrenderertext.h"
-#include "gtksourceview-i18n.h"
 
 /**
  * SECTION:gutterrenderertext
@@ -35,14 +31,14 @@
  * #GtkSourceGutter.
  */
 
-struct _GtkSourceGutterRendererTextPrivate
+typedef struct
 {
-	gchar *text;
-
+	gchar       *text;
 	PangoLayout *cached_layout;
-
-	guint is_markup : 1;
-};
+	GdkRGBA      cached_color;
+	gsize        text_len;
+	guint        is_markup : 1;
+} GtkSourceGutterRendererTextPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (GtkSourceGutterRendererText, gtk_source_gutter_renderer_text, GTK_SOURCE_TYPE_GUTTER_RENDERER)
 
@@ -50,168 +46,89 @@ enum
 {
 	PROP_0,
 	PROP_MARKUP,
-	PROP_TEXT
+	PROP_TEXT,
+	N_PROPS
 };
 
 static void
-gutter_renderer_text_begin (GtkSourceGutterRenderer *renderer,
-			    cairo_t                 *cr,
-			    GdkRectangle            *background_area,
-			    GdkRectangle            *cell_area,
-			    GtkTextIter             *start,
-			    GtkTextIter             *end)
+gtk_source_gutter_renderer_text_begin (GtkSourceGutterRenderer *renderer,
+                                       GtkSourceGutterLines    *lines)
 {
+
 	GtkSourceGutterRendererText *text = GTK_SOURCE_GUTTER_RENDERER_TEXT (renderer);
-	GtkTextView *view;
+	GtkSourceGutterRendererTextPrivate *priv = gtk_source_gutter_renderer_text_get_instance_private (text);
 
-	view = gtk_source_gutter_renderer_get_view (renderer);
+	GTK_SOURCE_GUTTER_RENDERER_CLASS (gtk_source_gutter_renderer_text_parent_class)->begin (renderer, lines);
 
-	g_clear_object (&text->priv->cached_layout);
-	text->priv->cached_layout = gtk_widget_create_pango_layout (GTK_WIDGET (view), NULL);
-
-	if (GTK_SOURCE_GUTTER_RENDERER_CLASS (gtk_source_gutter_renderer_text_parent_class)->begin != NULL)
-	{
-		GTK_SOURCE_GUTTER_RENDERER_CLASS (gtk_source_gutter_renderer_text_parent_class)->begin (renderer,
-													cr,
-													background_area,
-													cell_area,
-													start,
-													end);
-	}
+	g_clear_object (&priv->cached_layout);
+	priv->cached_layout = gtk_widget_create_pango_layout (GTK_WIDGET (renderer), NULL);
 }
 
 static void
-center_on (GtkTextView  *view,
-           GdkRectangle *cell_area,
-           GtkTextIter  *iter,
-           gint          width,
-           gint          height,
-           gfloat        xalign,
-           gfloat        yalign,
-           gint         *x,
-           gint         *y)
-{
-	GdkRectangle location;
-
-	gtk_text_view_get_iter_location (view, iter, &location);
-
-	*x = cell_area->x + (cell_area->width - width) * xalign;
-	*y = cell_area->y + (location.height - height) * yalign;
-}
-
-static void
-gutter_renderer_text_draw (GtkSourceGutterRenderer      *renderer,
-			   cairo_t                      *cr,
-			   GdkRectangle                 *background_area,
-			   GdkRectangle                 *cell_area,
-			   GtkTextIter                  *start,
-			   GtkTextIter                  *end,
-			   GtkSourceGutterRendererState  state)
+gtk_source_gutter_renderer_text_snapshot_line (GtkSourceGutterRenderer *renderer,
+                                               GtkSnapshot             *snapshot,
+                                               GtkSourceGutterLines    *lines,
+                                               guint                    line)
 {
 	GtkSourceGutterRendererText *text = GTK_SOURCE_GUTTER_RENDERER_TEXT (renderer);
-	GtkTextView *view;
+	GtkSourceGutterRendererTextPrivate *priv = gtk_source_gutter_renderer_text_get_instance_private (text);
+	PangoLayout *layout;
+	gfloat x;
+	gfloat y;
 	gint width;
 	gint height;
-	gfloat xalign;
-	gfloat yalign;
-	GtkSourceGutterRendererAlignmentMode mode;
-	gint x = 0;
-	gint y = 0;
-	GtkStyleContext *context;
 
-	/* Chain up to draw background */
-	if (GTK_SOURCE_GUTTER_RENDERER_CLASS (gtk_source_gutter_renderer_text_parent_class)->draw != NULL)
+	if (priv->text == NULL || priv->text_len == 0)
 	{
-		GTK_SOURCE_GUTTER_RENDERER_CLASS (gtk_source_gutter_renderer_text_parent_class)->draw (renderer,
-												       cr,
-												       background_area,
-												       cell_area,
-												       start,
-												       end,
-												       state);
+		return;
 	}
 
-	view = gtk_source_gutter_renderer_get_view (renderer);
+	layout = priv->cached_layout;
 
-	if (text->priv->is_markup)
+	if (priv->is_markup)
 	{
-		pango_layout_set_markup (text->priv->cached_layout,
-		                         text->priv->text,
-		                         -1);
+		pango_layout_set_markup (layout,
+		                         priv->text,
+		                         priv->text_len);
 	}
 	else
 	{
-		pango_layout_set_text (text->priv->cached_layout,
-		                       text->priv->text,
-		                       -1);
+		pango_layout_set_text (layout,
+		                       priv->text,
+		                       priv->text_len);
 	}
 
-	pango_layout_get_pixel_size (text->priv->cached_layout, &width, &height);
+	pango_layout_get_pixel_size (layout, &width, &height);
 
-	gtk_source_gutter_renderer_get_alignment (renderer,
-	                                          &xalign,
-	                                          &yalign);
+	gtk_source_gutter_renderer_align_cell (renderer,
+	                                       line,
+	                                       width,
+	                                       height,
+	                                       &x,
+	                                       &y);
 
-	/* Avoid calculations if we don't wrap text */
-	if (gtk_text_view_get_wrap_mode (view) == GTK_WRAP_NONE)
+	gtk_snapshot_render_layout (snapshot,
+	                            gtk_widget_get_style_context (GTK_WIDGET (text)),
+	                            x,
+	                            y,
+	                            layout);
+
+	if (priv->is_markup)
 	{
-		mode = GTK_SOURCE_GUTTER_RENDERER_ALIGNMENT_MODE_CELL;
+		pango_layout_set_attributes (layout, NULL);
 	}
-	else
-	{
-		mode = gtk_source_gutter_renderer_get_alignment_mode (renderer);
-	}
-
-	switch (mode)
-	{
-		case GTK_SOURCE_GUTTER_RENDERER_ALIGNMENT_MODE_CELL:
-			x = cell_area->x + (cell_area->width - width) * xalign;
-			y = cell_area->y + (cell_area->height - height) * yalign;
-			break;
-
-		case GTK_SOURCE_GUTTER_RENDERER_ALIGNMENT_MODE_FIRST:
-			center_on (view,
-			           cell_area,
-			           start,
-			           width,
-			           height,
-			           xalign,
-			           yalign,
-			           &x,
-			           &y);
-			break;
-
-		case GTK_SOURCE_GUTTER_RENDERER_ALIGNMENT_MODE_LAST:
-			center_on (view,
-			           cell_area,
-			           end,
-			           width,
-			           height,
-			           xalign,
-			           yalign,
-			           &x,
-			           &y);
-			break;
-
-		default:
-			g_assert_not_reached ();
-	}
-
-	context = gtk_widget_get_style_context (GTK_WIDGET (view));
-	gtk_render_layout (context, cr, x, y, text->priv->cached_layout);
 }
 
 static void
-gutter_renderer_text_end (GtkSourceGutterRenderer *renderer)
+gtk_source_gutter_renderer_text_end (GtkSourceGutterRenderer *renderer)
 {
+
 	GtkSourceGutterRendererText *text = GTK_SOURCE_GUTTER_RENDERER_TEXT (renderer);
+	GtkSourceGutterRendererTextPrivate *priv = gtk_source_gutter_renderer_text_get_instance_private (text);
 
-	g_clear_object (&text->priv->cached_layout);
+	GTK_SOURCE_GUTTER_RENDERER_CLASS (gtk_source_gutter_renderer_text_parent_class)->end (renderer);
 
-	if (GTK_SOURCE_GUTTER_RENDERER_CLASS (gtk_source_gutter_renderer_text_parent_class)->end != NULL)
-	{
-		GTK_SOURCE_GUTTER_RENDERER_CLASS (gtk_source_gutter_renderer_text_parent_class)->end (renderer);
-	}
+	g_clear_object (&priv->cached_layout);
 }
 
 static void
@@ -221,13 +138,33 @@ measure_text (GtkSourceGutterRendererText *renderer,
               gint                        *width,
               gint                        *height)
 {
-	GtkTextView *view;
+	GtkSourceView *view;
 	PangoLayout *layout;
 
+	if (width != NULL)
+	{
+		*width = 0;
+	}
+
+	if (height != NULL)
+	{
+		*height = 0;
+	}
+
 	view = gtk_source_gutter_renderer_get_view (GTK_SOURCE_GUTTER_RENDERER (renderer));
 
+	if (view == NULL)
+	{
+		return;
+	}
+
 	layout = gtk_widget_create_pango_layout (GTK_WIDGET (view), NULL);
 
+	if (layout == NULL)
+	{
+		return;
+	}
+
 	if (markup != NULL)
 	{
 		pango_layout_set_markup (layout, markup, -1);
@@ -290,13 +227,54 @@ gtk_source_gutter_renderer_text_measure_markup (GtkSourceGutterRendererText *ren
 	measure_text (renderer, markup, NULL, width, height);
 }
 
+static void
+gtk_source_gutter_renderer_text_real_measure (GtkWidget      *widget,
+                                              GtkOrientation  orientation,
+                                              int             for_size,
+                                              int            *minimum,
+                                              int            *natural,
+                                              int            *minimum_baseline,
+                                              int            *natural_baseline)
+{
+	GtkSourceGutterRendererText *renderer = GTK_SOURCE_GUTTER_RENDERER_TEXT (widget);
+	GtkSourceGutterRendererTextPrivate *priv = gtk_source_gutter_renderer_text_get_instance_private (renderer);
+
+	*minimum = 0;
+	*natural = 0;
+	*minimum_baseline = -1;
+	*natural_baseline = -1;
+
+	if (orientation == GTK_ORIENTATION_HORIZONTAL)
+	{
+		gint xpad = gtk_source_gutter_renderer_get_xpad (GTK_SOURCE_GUTTER_RENDERER (renderer));
+		gint width = 0;
+		gint height = 0;
+
+		if (priv->text != NULL)
+		{
+			if (priv->is_markup)
+			{
+				measure_text (renderer, priv->text, NULL, &width, &height);
+			}
+			else
+			{
+				measure_text (renderer, NULL, priv->text, &width, &height);
+			}
+		}
+
+		*natural = *minimum = width + 2 * xpad;
+	}
+
+}
+
 static void
 gtk_source_gutter_renderer_text_finalize (GObject *object)
 {
 	GtkSourceGutterRendererText *renderer = GTK_SOURCE_GUTTER_RENDERER_TEXT (object);
+	GtkSourceGutterRendererTextPrivate *priv = gtk_source_gutter_renderer_text_get_instance_private (renderer);
 
-	g_free (renderer->priv->text);
-	g_clear_object (&renderer->priv->cached_layout);
+	g_clear_pointer (&priv->text, g_free);
+	g_clear_object (&priv->cached_layout);
 
 	G_OBJECT_CLASS (gtk_source_gutter_renderer_text_parent_class)->finalize (object);
 }
@@ -307,10 +285,22 @@ set_text (GtkSourceGutterRendererText *renderer,
           gint                         length,
           gboolean                     is_markup)
 {
-	g_free (renderer->priv->text);
+	GtkSourceGutterRendererTextPrivate *priv = gtk_source_gutter_renderer_text_get_instance_private (renderer);
 
-	renderer->priv->text = length >= 0 ? g_strndup (text, length) : g_strdup (text);
-	renderer->priv->is_markup = is_markup;
+	g_free (priv->text);
+
+	if (text == NULL)
+	{
+		priv->text_len = 0;
+		priv->text = NULL;
+		priv->is_markup = FALSE;
+	}
+	else
+	{
+		priv->text_len = length >= 0 ? length : strlen (text);
+		priv->text = g_strndup (text, priv->text_len);
+		priv->is_markup = !!is_markup;
+	}
 }
 
 static void
@@ -319,9 +309,7 @@ gtk_source_gutter_renderer_text_set_property (GObject      *object,
                                               const GValue *value,
                                               GParamSpec   *pspec)
 {
-	GtkSourceGutterRendererText *renderer;
-
-	renderer = GTK_SOURCE_GUTTER_RENDERER_TEXT (object);
+	GtkSourceGutterRendererText *renderer = GTK_SOURCE_GUTTER_RENDERER_TEXT (object);
 
 	switch (prop_id)
 	{
@@ -343,17 +331,16 @@ gtk_source_gutter_renderer_text_get_property (GObject    *object,
                                               GValue     *value,
                                               GParamSpec *pspec)
 {
-	GtkSourceGutterRendererText *renderer;
-
-	renderer = GTK_SOURCE_GUTTER_RENDERER_TEXT (object);
+	GtkSourceGutterRendererText *renderer = GTK_SOURCE_GUTTER_RENDERER_TEXT (object);
+	GtkSourceGutterRendererTextPrivate *priv = gtk_source_gutter_renderer_text_get_instance_private (renderer);
 
 	switch (prop_id)
 	{
 		case PROP_MARKUP:
-			g_value_set_string (value, renderer->priv->is_markup ? renderer->priv->text : NULL);
+			g_value_set_string (value, priv->is_markup ? priv->text : NULL);
 			break;
 		case PROP_TEXT:
-			g_value_set_string (value, !renderer->priv->is_markup ? renderer->priv->text : NULL);
+			g_value_set_string (value, !priv->is_markup ? priv->text : NULL);
 			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -365,16 +352,18 @@ static void
 gtk_source_gutter_renderer_text_class_init (GtkSourceGutterRendererTextClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 	GtkSourceGutterRendererClass *renderer_class = GTK_SOURCE_GUTTER_RENDERER_CLASS (klass);
 
 	object_class->finalize = gtk_source_gutter_renderer_text_finalize;
-
 	object_class->get_property = gtk_source_gutter_renderer_text_get_property;
 	object_class->set_property = gtk_source_gutter_renderer_text_set_property;
 
-	renderer_class->begin = gutter_renderer_text_begin;
-	renderer_class->draw = gutter_renderer_text_draw;
-	renderer_class->end = gutter_renderer_text_end;
+	widget_class->measure = gtk_source_gutter_renderer_text_real_measure;
+
+	renderer_class->begin = gtk_source_gutter_renderer_text_begin;
+	renderer_class->end = gtk_source_gutter_renderer_text_end;
+	renderer_class->snapshot_line = gtk_source_gutter_renderer_text_snapshot_line;
 
 	g_object_class_install_property (object_class,
 	                                 PROP_MARKUP,
@@ -396,9 +385,9 @@ gtk_source_gutter_renderer_text_class_init (GtkSourceGutterRendererTextClass *kl
 static void
 gtk_source_gutter_renderer_text_init (GtkSourceGutterRendererText *self)
 {
-	self->priv = gtk_source_gutter_renderer_text_get_instance_private (self);
+	GtkSourceGutterRendererTextPrivate *priv = gtk_source_gutter_renderer_text_get_instance_private (self);
 
-	self->priv->is_markup = TRUE;
+	priv->is_markup = TRUE;
 }
 
 /**
