@@ -1,8 +1,7 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8; coding: utf-8 -*-
- *
+/*
  * This file is part of GtkSourceView
  *
- * Copyright (C) 2015 - Paolo Borelli <pborelli@gnome.org>
+ * Copyright 2015 - Paolo Borelli <pborelli@gnome.org>
  *
  * GtkSourceView is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,9 +17,7 @@
  * along with this library; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "config.h"
 
 #include "gtksourceversion.h"
 
@@ -114,5 +111,5 @@ gtk_source_check_version (guint major,
                           guint minor,
                           guint micro)
 {
-	return GTK_SOURCE_CHECK_VERSION (major, (int)minor, micro);
+	return GTK_SOURCE_CHECK_VERSION (major, minor, micro);
 }
